class Hermes::Admin::NudProvidersController < Hermes::Admin::BaseController
  before_action :authorize_users, :check_adminrole
  before_action :set_nud_provider, only: %i[edit update destroy]

  def index
    @providers =  Hermes::NudProvider.readable_by(current_user).paginate(page: params[:page])
  end

  def edit; end

  def new
    @provider = Hermes::NudProvider.new
  end

  def create
    @provider = Hermes::NudProvider.new(nud_params)
    @provider.user_id = current_user.id
    if @provider.save
      flash[:notice] = "Créé avec succès."
      redirect_to admin_nud_providers_path
    else
      flash[:error] = @provider.errors.full_messages.to_sentence
      render :new
    end
  end

  def update
    if @provider.update(nud_params)
      flash[:notice] = "Mise à jour effectuée"
      redirect_to admin_nud_providers_path
    else
      flash[:error] = @provider.errors.full_messages.to_sentence
      render :edit
    end
  end

  def destroy
    if @provider.destroy
      flash[:notice] = "Fournisseur supprimé"
      redirect_to admin_nud_providers_path
    else
      flash[:error] = @provider.errors.full_messages.to_sentence
      render :show
    end
  end

  private

  def nud_params
    params.require(:nud_provider).permit(:name, :url, :department_codes_text)
  end

  def set_nud_provider
    @provider = Hermes::NudProvider.find(params[:id])
  end
end