module Hermes
  module Api
    module Dmo
      class PicturesController < Hermes::Api::Dmo::ApplicationController
        before_action :set_housing_offer
        before_action :set_picture, only: [:update, :destroy]
        before_action :check_has_access, only: [:create, :update, :destroy]
        before_action :check_upload_file, only: [:create, :update]

        MAX_PICTURE_NUMBER = 20
        MUTATE_PARAMS = %i[]

        def mutate_params
          MUTATE_PARAMS
        end 
  
        def create
          user_id = @user.id
          @instance = Hermes::Picture.new(user_id: user_id, housing_offer_id: @housing_offer.id, readers:@housing_offer.readers, writers: @housing_offer.writers)

          blob = @file.read
          @instance.set_blob(blob)

          return respond_with_errors(@instance) unless @instance.save
          render(jsonapi: @instance, serializer: Hermes::PictureSerializer, status: :created)
        end
      
        def update
          return render_json_error(:unprocessable_entity, :not_writers) unless @instance.reload && @instance.writable_by?(@user)

          blob = @file.read
          @instance.set_blob(blob)

          return respond_with_errors(@instance) unless  @instance.save
          render(jsonapi: @instance, serializer: Hermes::PictureSerializer)
        end

        private 

        def set_picture
          return render_json_error :not_found, :picture_not_found  unless @instance = Hermes::Picture.find_by(id: params[:id])
        end
        
        def set_housing_offer
          return render_json_error :not_found, :housing_offer_not_found  unless @housing_offer = Hermes::HousingOffer.find_by(id: params[:housing_offer_id])
        end

        def check_has_access
          return render_json_error :unauthorized, :no_access_to_offer unless is_admin_or_super_admin || @housing_offer.readable_by?(@user)
        end 
        
        def check_upload_file
          return render_json_error :unprocessable_entity, :no_file_in_the_request unless @file = params[:file]
          return render_json_error :unprocessable_entity, :number_of_pictures_exeded if @housing_offer&.ordered_picture_ids && @housing_offer.ordered_picture_ids.length >= MAX_PICTURE_NUMBER
          
          begin 
            Magick::Image.ping(@file.path).first.nil?
          rescue Magick::ImageMagickError
            return render_json_error :unsupported_media_type, :not_an_image 
          end
          
        end 
      end
    end
  end
end