require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::AccountingsController, :type => :api do
  routes { Hermes::Engine.routes }
  include Devise::Test::ControllerHelpers
  
  let!(:admin) { create :administrator, auth_roles: ['superadministrateur'] }

  describe 'GET index' do
    let!(:request1) { create :accounting, target: 'request', year: 2019 }
    let!(:request2) { create :accounting, target: 'request', year: 2020 }
    let!(:request3) { create :accounting, target: 'request', year: 2020 }
    let!(:offer1) { create :accounting, target: 'offer', year: 2020 }
    let!(:offer2) { create :accounting, target: 'offer', year: 2021 }

    before do
      authenticated_header(request, admin)
    end

    it 'returns a valid json' do
      get 'index'
      expect(response).to have_http_status 200
      expect(json['data'].count).to eq 5
      expect(json['meta']['pagination']['total_objects']).to eq 5
      expect(json['data'].map { |d| d['id'] }).to contain_exactly(request1.id, request2.id, request3.id, offer1.id, offer2.id)
    end

    it 'filters by target' do
      get 'index', target: 'offer'
      expect(response).to have_http_status 200
      expect(json['data'].count).to eq 2
      expect(json['data'].map { |d| d['id'] }).to contain_exactly(offer1.id, offer2.id)
    end

    it 'filters by year' do
      get 'index', year: 2020
      expect(response).to have_http_status 200
      expect(json['data'].count).to eq 3
      expect(json['data'].map { |d| d['id'] }).to contain_exactly(request2.id, request3.id, offer1.id)
    end
  end
end

describe Hermes::Api::Dmo::AccountingsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  require_relative '../../swagger_helper'
  tags = 'API_DMO_ACCOUNTINGS'
  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes

  path '/api/dmo/accountings' do
    get 'Retrieves all accountings' do
      tags tags
      consumes 'application/json'
      produces 'application/json'

      let!(:admin) { create :administrator, auth_roles: ['superadministrateur'] }

      Hermes::Api::Dmo::AccountingsController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "" }
        run_test!
      end 

      response '200', 'accountings found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        run_test!
        
      end
    end
  end
end