require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::UsersController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers

  describe 'set user and errors' do 
    let!(:user) { create :user }

    it 'can connect' do
      authenticated_header(request, user)
      get 'show', id: user.id
      expect(response.status).to eq(200)
    end 

    it 'invalidated_token' do 
      token = authenticated_header(request, user)
      Hermes::UserInvalidatedToken.create(user_id: user.id, token: token, expiring_at: Time.now + 1.hours)
      get 'show', id: user.id
      expect(response.status).to eq(401)
      expect(json['errors'].first).to eq('User token invalidated')
    end 

    it 'no bearer token' do 
      get 'show', id: user.id
      expect(response.status).to eq(401)
      expect(json['errors'].first).to eq('Not Authenticated')
    end 

    it 'token expired' do 
      authenticated_header(request, user)

      Timecop.freeze(Time.now + 5.hours) 
      get 'show', id: user.id
      expect(response.status).to eq(401)
      expect(json['errors'].first).to eq('Token expired')
      Timecop.return
    end 

    it 'when using X-auth-user-token' do 
      request.headers.merge!('X-auth-user-token': user.authentication_token)

      get 'show', id: user.id
      expect(response.status).to eq(200)
    end 

    it 'when using wrong X-auth-user-token' do 
      request.headers.merge!('X-auth-user-token': 'invalid_token')

      get 'show', id: user.id
      expect(response.status).to eq(401)
      expect(json['errors'].first).to eq('Wrong token')
    end 

  end 
end